@@ -1,12 +1,13 @@
-
-import functools
+"""
+String operate functions.
+"""
+import itertools
 import random
 import string
-import os
 import binascii
 import base64
-import bizerror
 import typing
+import bizerror
 import math
 import re
 import uuid
@@ -25,32 +26,56 @@ YI = WAN * WAN
 
 default_encodings = ["utf8", "gb18030"]
 
-
-
 def random_string(length, choices=string.ascii_letters):
+    """Generates a random string of specified length using specified characters.
+
+    In [3]: strutils.random_string(8)
+    Out[3]: 'wJhsaYVq'
+
+    In [4]: strutils.random_string(32)
+    Out[4]: 'JtYNblzfEFYdgwcTjNiorwucrlHuIeTQ'
+
+    In [5]: import string
+
+    In [8]: strutils.random_string(32, choices=string.ascii_letters+string.punctuation)
+    Out[8]: "]ivN^F]v#jPraNLC:F?<}:!'}aox=lKY"
+
+    In [9]: strutils.random_string(32, choices=string.ascii_letters+string.punctuation)
+    Out[9]: 'Flx,TOkDV_g&CsHq#l`RZ:(J#eVxENYJ'
+    """
     return "".join(random.choices(choices, k=length))
 
-def char_force_to_int(value):
-    if value is None:
+def char_force_to_int(char):
+    if callable(char):
+        char = char()
+    if char is None:
         return None
-    if isinstance(value, int):
-        return value
-    return ord(value)
+    if isinstance(char, int):
+        return char
+    return ord(char)
 
-def force_bytes(value, encoding="utf-8"):
+def force_bytes(value, encoding=None):
+    if callable(value):
+        value = value()
     if value is None:
         return None
+    if isinstance(value, bytes):
+        return value
+    encoding = encoding or "utf-8"
     if isinstance(value, str):
-        encoding = encoding or "utf-8"
         return value.encode(encoding)
     else:
-        return value
+        return str(value).encode(encoding)
 
 def force_text(value, encoding=None):
+    if callable(value):
+        value = value()
     if value is None:
         return None
     if isinstance(value, str):
         return value
+    if not isinstance(value, bytes):
+        return str(value)
     if not encoding:
         encodings = default_encodings
     elif isinstance(encoding, (set, list, tuple)):
@@ -65,6 +90,8 @@ def force_text(value, encoding=None):
     raise UnicodeDecodeError()
 
 def force_int(value):
+    if callable(value):
+        value = value()
     if value is None:
         return None
     if isinstance(value, int):
@@ -79,6 +106,8 @@ def force_int(value):
     return int(value)
 
 def force_float(value):
+    if callable(value):
+        value = value()
     if value is None:
         return None
     if isinstance(value, float):
@@ -93,6 +122,8 @@ def force_float(value):
     return float(value)
 
 def force_numberic(value):
+    if callable(value):
+        value = value()
     if value is None:
         return None
     if isinstance(value, (int, float, Decimal)):
@@ -106,22 +137,48 @@ def force_numberic(value):
         return int(value)
 
 def wholestrip(text):
-    """Remove all white spaces in text. White spaces are ' \t\n\r\x0b\x0c\u3000'.
+    """Remove all white spaces in text. White spaces are ' \\t\\n\\r\\x0b\\x0c\\u3000'. 
+    
+    Note: 
+    
+    It's NOT only trim the left and right white characters, 
+    but also remove white characters in the middle of the text.
     """
-    for space in string.whitespace + u'\u3000':
+    if text is None:
+        return None
+    for space in string.whitespace + "\u3000":
         text = text.replace(space, "")
     return text
 
-def split2(text, sep=None):
-    """Split string into 2 segments.
+def split2(text, seps=None):
+    """Split string into two parts.
     """
-    ss = text.split(sep, 1)
+    if text is None:
+        return None
+    if not seps:
+        ss = text.split(maxsplit=1)
+    else:
+        if not isinstance(seps, (list, set, tuple)):
+            seps = [seps]
+        min_index = len(text)
+        min_sep = None
+        for sep in seps:
+            try:
+                index = text.index(sep)
+                if index < min_index:
+                    min_index = index
+                    min_sep = sep
+            except ValueError:
+                pass
+        ss = text.split(min_sep, maxsplit=1)
+    if len(ss) == 0:
+        ss.append("")
     if len(ss) == 1:
         ss.append("")
     return ss
 
 def split(text, seps, strip=False):
-    """seps is a list of string, all sep in the seps are treated as delimiter.
+    """Split string with a list of sperators. seps is a list of string, all sep in the seps are treated as delimiter.
     """
     if not text:
         return []
@@ -282,7 +339,20 @@ def is_chinese_character(c):
         return True
     return False
 
-def binarify(data):
+def binarify(data:bytes) -> str:
+    """Turn bytes into binary string. Similar to binascii.hexlify(), but using binary instread hex.
+    
+    Examples:
+
+    In [11]: strutils.binarify(b'0')
+    Out[11]: '00110000'
+
+    In [12]: strutils.binarify(b'a')
+    Out[12]: '01100001'
+
+    In [13]: strutils.binarify(b'hello')
+    Out[13]: '0110100001100101011011000110110001101111'
+    """
     if isinstance(data, str):
         data = data.encode("utf-8")
     return "".join(["{:08b}".format(x) for x in data])
@@ -325,7 +395,46 @@ def substrings(value, lengths=None):
             subs.add(sub)
     return subs
 
-def combinations(values, length):
+def combinations2(values: typing.List[str], length:int) -> typing.List[str]:
+    """同combinations，使用了itertools.product实现，简化了实现逻辑。但性能仅为combinations的一半。仅供参考。
+    """
+    results = set()
+    min_length = min([len(x) for x in values])
+    repeat = math.ceil(length / min_length * 2)
+    for vs in itertools.product(values, repeat=repeat):
+        line = "".join(vs)
+        line_length = len(line)
+        max_start_index = line_length - length
+        for index in range(max_start_index+1):
+            word = line[index:index+length]
+            results.add(word)
+    return results
+
+def combinations(values: typing.List[str], length:int) -> typing.List[str]:
+    """取values中各字符串的任意连接（可重复取），在组成的新字符串中，取任意指定长度子串所形成的集合。
+
+    例如：
+
+    In [96]: strutils.combinations(['abc', 'xyz'], 5)
+    Out[96]:
+    {'abcab',
+    'abcxy',
+    'bcabc',
+    'bcxyz',
+    'cabca',
+    'cabcx',
+    'cxyza',
+    'cxyzx',
+    'xyzab',
+    'xyzxy',
+    'yzabc',
+    'yzxyz',
+    'zabca',
+    'zabcx',
+    'zxyza',
+    'zxyzx'}
+
+    """
     values = list(values)
     min_length = min([len(x) for x in values])
     max_length = max([len(x) for x in values])
@@ -341,7 +450,6 @@ def combinations(values, length):
         repeat += 1
     check_length = length + 2 * max_length - 1
     good_words = set()
-    words = set()
     short_words = set(values)
     for i in range(repeat):
         new_words = set()
@@ -430,18 +538,20 @@ def captital_number(value, yuan="圆", digits="零壹贰叁肆伍陆柒捌玖",
 
     return result
 
-def clean(value, clean_chars):
-    if not isinstance(clean_chars, set):
-        clean_chars = set(clean_chars)
+def clean(value, keep_chars):
+    """Clean the string value and only keep characters in keep_chars.
+    """
+    if not isinstance(keep_chars, set):
+        keep_chars = set(keep_chars)
     value2 = ""
     for c in value:
-        if c in clean_chars:
+        if c in keep_chars:
             value2 += c
     return value2
 
-do_clean = clean
+do_clean = clean # Alias of clean
 
-def camel(value, clean=False, clean_chars=string.ascii_letters + string.digits, lower_first=False):
+def camel(value, clean=False, keep_chars=string.ascii_letters + string.digits, lower_first=False):
     if value is None:
         return None
     words = []
@@ -464,8 +574,8 @@ def camel(value, clean=False, clean_chars=string.ascii_letters + string.digits,
     if non_word:
         words.append(non_word)
     if clean:
-        clean_chars = set(clean_chars)
-        result = "".join([do_clean(x, clean_chars) for x in words])
+        keep_chars = set(keep_chars)
+        result = "".join([do_clean(x, keep_chars) for x in words])
     else:
         result = "".join(words)
     if len(result) < 1:
@@ -484,6 +594,8 @@ def none_to_empty_string(value):
         return value
 
 def strip_string(value):
+    """If value is str, then do string strip, or else returns the value itself.
+    """
     if isinstance(value, str):
         return value.strip()
     else:
@@ -523,12 +635,12 @@ def format_with_mapping(template, mapping, *args, **kwargs):
 
 
 default_quotes = [
-    '"""',
+    '"""', # quote left and quote right are the same
     "'''",
     '"',
     "'",
     "`",
-    ('“', '”'),
+    ('“', '”'), # quote left and quote right are NOT the same
     ("‘", "’"),
     ("『", "』"),
     ("「", "」"),
@@ -538,6 +650,8 @@ default_quotes = [
     ]
 
 def unquote(value, quotes=None):
+    """Trim one level quote. If not quoted, do nothing.
+    """
     quotes = quotes or default_quotes
     for quote_pair in quotes:
         if isinstance(quote_pair, str):
@@ -548,29 +662,35 @@ def unquote(value, quotes=None):
 
 
 def is_uuid(value, allow_bad_characters=False):
+    """Test if the value is UUID typed or UUID liked str.
+    
+    If allow_bad_characters=True, 
+    treat c1fd56f3-bd79-42ed-a45c-d711c4032bag liked string as UUID,
+    even it contains NON hex character(the last character g is NOT a hex digist).
+    """
     if isinstance(value, uuid.UUID):
         return True
     if isinstance(value, bytes):
         try:
-            value2 = uuid.UUID(bytes=value)
+            _ = uuid.UUID(bytes=value)
             return True
         except ValueError:
             return False
     elif isinstance(value, (tuple, list)):
         try:
-            value2 = uuid.UUID(fields=value)
+            _ = uuid.UUID(fields=value)
             return True
         except ValueError:
             return False
     elif isinstance(value, int):
         try:
-            value2 = uuid.UUID(int=value)
+            _ = uuid.UUID(int=value)
             return True
         except ValueError:
             return False
     else:
         try:
-            value2 = uuid.UUID(value)
+            _ = uuid.UUID(value)
             return True
         except ValueError:
             if allow_bad_characters:
@@ -604,16 +724,68 @@ def remove_suffix(thestring, suffix):
     else:
         return thestring
 
-def encodable(value, encoding="utf-8"):
+def encodable(value:str, encoding:str="utf-8") -> bool:
+    """Test if the string value can be encoded by special encoding.
+
+    Examples:
+
+    In [11]: strutils.encodable('hello') # ascii letters can be encoded by utf-8 encoding.
+    Out[11]: True
+
+    In [12]: strutils.encodable('测试') # 中文 测试 can be encoded by utf-8 encoding.
+    Out[12]: True
+
+    In [13]: strutils.encodable('测试', encoding='big5') # 中文 测试 can NOT be encoded by big5 encoding.
+    Out[13]: False
+
+    """
     try:
         value.encode(encoding)
         return True
     except UnicodeEncodeError:
         return False
 
-def decodable(value, encoding="utf-8"):
+def decodable(value:bytes, encoding:str="utf-8") -> bool:
+    """Test if the bytes value can be decoded by special encoding.
+
+    Examples:
+
+    In [2]: strutils.decodable('测试'.encode('gbk')) # string encoded by gbk can not be decoded by utf-8
+    Out[2]: False
+
+    In [3]: strutils.decodable('测试'.encode('gbk'), encoding='gbk') # string encoded by gbk can be decoded by gbk
+    Out[3]: True
+
+    """
     try:
         value.decode(encoding)
         return True
     except UnicodeDecodeError:
         return False
+
+def chunk(value:str, size:int) -> typing.List[str]:
+    """Split string value into chunks. Chunk size must be greater than 0.
+
+    Examples:
+
+    In [33]: strutils.chunk('hello', 3)
+    Out[33]: ['hel', 'lo']
+
+    In [34]: strutils.chunk('hello', 6)
+    Out[34]: ['hello']
+
+    In [35]: strutils.chunk('hello', 5)
+    Out[35]: ['hello']
+
+    """
+    if size < 1:
+        raise ValueError("chunk size must be greater than 0...")
+    if value is None:
+        return []
+    chunks = []
+    length = len(value)
+    start = 0
+    while start < length:
+        chunks.append(value[start:start+size])
+        start += size
+    return chunks
