@@ -186,7 +186,7 @@ def smart_cast(type, value, field_name=None):
     if type in TYPE_CASTERS:
         type_func = TYPE_CASTERS[type]
     elif callable(type):
-        type_func = type_func
+        type_func = type
     else:
         raise bizerror.NotSupportedTypeToCast(field=field_name, type=str(type(value)), value=str(value))
     try:
